const glob = require('glob');
const Promise = require('bluebird');
const extractSnippets = require('./extractSnippets');

const decorateJSParserWithVueSupport = jsParser => {
  const originalParseString = jsParser.parseString;

  jsParser.parseString = function(string, fileName, options = {}) {
    return Promise.resolve(originalParseString.call(this, string, fileName, options));
  };

  jsParser.parseFile = function(fileName, options = {}) {
    return extractSnippets(fileName).then(snippets => {
      snippets.forEach(snippet => {
        originalParseString.call(
          this,
          snippet.code,
          fileName,
          Object.assign({}, options, {
            lineNumberStart: snippet.line,
          })
        );
      });
      return jsParser;
    });
  };

  jsParser.parseFilesGlob = function(pattern, globOptions, options) {
    let files;
    try {
      files = glob.sync(pattern, globOptions);
    } catch (e) {
      return Promise.reject(e);
    }

    return Promise.map(files, file => jsParser.parseFile(file, options), { concurrency: 1 }).then(
      () => jsParser
    );
  };

  return jsParser;
};

module.exports = decorateJSParserWithVueSupport;
